//
// SkinnedDialog.cpp
//

#include "stdafx.h"
#include "SkinnedDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////

IMPLEMENT_DYNAMIC( CSkinnedDialog, CDialog )

/////////////////////////////////////////////////////////////////////////////

CSkinnedDialog::CSkinnedDialog( UINT nIDTemplate, CWnd* pParentWnd /*= NULL*/, UINT nIDSkin /*= IDB_DEFAULTSKIN*/, COLORREF clrSkinBkgrd /*= RGB( 223, 248, 255 )*/ ) :
	CDialog( nIDTemplate, pParentWnd ),
	m_pswcoCompanionObj( NULL ),
	m_bProvideDefIcon( TRUE )
{
		// save the construction parameters

		m_nIDSkin = nIDSkin;
		m_clrSkinBkgrd = clrSkinBkgrd;
}

CSkinnedDialog::~CSkinnedDialog ()
{
	if ( m_pswcoCompanionObj )
		delete m_pswcoCompanionObj;
}

BEGIN_MESSAGE_MAP(CSkinnedDialog, CDialog)
	//{{AFX_MSG_MAP(CSkinnedDialog)
	ON_WM_PAINT()
	ON_WM_ERASEBKGND()
	ON_WM_NCPAINT()
	ON_WM_GETMINMAXINFO()
	ON_WM_SIZE()
	ON_WM_NCHITTEST()
	ON_WM_NCACTIVATE()
	ON_WM_NCLBUTTONDOWN()
	ON_WM_NCLBUTTONUP()
	ON_WM_NCRBUTTONDOWN()
	ON_WM_NCRBUTTONUP()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
    ON_WM_MOUSEMOVE()
	ON_WM_NCMOUSEMOVE()
	ON_WM_SETCURSOR()
	ON_WM_INITMENU()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_NCLBUTTONDBLCLK()
	ON_WM_CAPTURECHANGED()
	ON_WM_TIMER()
	ON_WM_CTLCOLOR()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CSkinnedDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	// check if we have to set the default icon

	if ( m_bProvideDefIcon )
	{
		HICON		hDefIcon = AfxGetApp()->LoadIcon( IDR_MAINFRAME );
		if ( hDefIcon )
		{
			SetIcon( hDefIcon, TRUE );
			SetIcon( hDefIcon, FALSE );
		}
	}

	// Initialize the companion object

	m_pswcoCompanionObj = new CSkinnedWndCompanionObj( this, m_nIDSkin, m_clrSkinBkgrd );
	if ( m_pswcoCompanionObj->Initialize() == FALSE )
		throw;

	// return to the caller

	return TRUE;
}

void CSkinnedDialog::OnPaint() 
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnPaint();
}

BOOL CSkinnedDialog::OnEraseBkgnd(CDC* pDC) 
{
	if ( m_pswcoCompanionObj )
		return m_pswcoCompanionObj->OnEraseBkgnd(pDC);
	else
		return TRUE;
}

void CSkinnedDialog::OnNcPaint() 
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnNcPaint();
}

void CSkinnedDialog::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI) 
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnGetMinMaxInfo( lpMMI );
	CDialog::OnGetMinMaxInfo( lpMMI );
}

void CSkinnedDialog::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize( nType, cx, cy );
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnSize( nType, cx, cy );
}

UINT CSkinnedDialog::OnNcHitTest(CPoint point) 
{
	if ( m_pswcoCompanionObj )
		return m_pswcoCompanionObj->OnNcHitTest( point );
	else
		return CDialog::OnNcHitTest( point );
}

BOOL CSkinnedDialog::OnNcActivate(BOOL bActive) 
{
	if ( m_pswcoCompanionObj )
		return m_pswcoCompanionObj->OnNcActivate( bActive );
	else
		return CDialog::OnNcActivate( bActive );
}

void CSkinnedDialog::OnNcLButtonDown(UINT nHitTest, CPoint point) 
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnNcLButtonDown(nHitTest, point);
	else
		CDialog::OnNcLButtonDown(nHitTest, point);
}

void CSkinnedDialog::OnNcLButtonUp(UINT nHitTest, CPoint point) 
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnNcLButtonUp(nHitTest, point);
	else
		CDialog::OnNcLButtonUp(nHitTest, point);
}

void CSkinnedDialog::OnNcRButtonDown(UINT nHitTest, CPoint point) 
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnNcRButtonDown(nHitTest, point);
	else
		CDialog::OnNcRButtonDown(nHitTest, point);
}

void CSkinnedDialog::OnNcRButtonUp(UINT nHitTest, CPoint point) 
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnNcRButtonUp(nHitTest, point);
	else
		CDialog::OnNcRButtonUp(nHitTest, point);
}

void CSkinnedDialog::OnLButtonDown( UINT nFlags, CPoint point )
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnLButtonDown( nFlags, point );
	else
		CDialog::OnLButtonDown( nFlags, point );
}

void CSkinnedDialog::OnLButtonUp( UINT nFlags, CPoint point )
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnLButtonUp( nFlags, point );
	else
		CDialog::OnLButtonUp( nFlags, point );
}

void CSkinnedDialog::OnRButtonDown( UINT nFlags, CPoint point )
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnRButtonDown( nFlags, point );
	else
		CDialog::OnRButtonDown( nFlags, point );
}

void CSkinnedDialog::OnRButtonUp( UINT nFlags, CPoint point )
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnRButtonUp( nFlags, point );
	else
		CDialog::OnRButtonUp( nFlags, point );
}

void CSkinnedDialog::OnMouseMove(UINT nFlags, CPoint point)
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnMouseMove( nFlags, point );
	else
		CDialog::OnMouseMove( nFlags, point );
}

void CSkinnedDialog::OnNcMouseMove(UINT nHitTest, CPoint point)
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnNcMouseMove( nHitTest, point );
	else
		CDialog::OnNcMouseMove( nHitTest, point );
}

BOOL CSkinnedDialog::OnSetCursor( CWnd* pWnd, UINT nHitTest, UINT message )
{
	if ( m_pswcoCompanionObj )
		return m_pswcoCompanionObj->OnSetCursor( pWnd, nHitTest, message );
	else
		return CDialog::OnSetCursor( pWnd, nHitTest, message );
}

void CSkinnedDialog::OnInitMenu( CMenu* pMenu )
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnInitMenu( pMenu );
	CDialog::OnInitMenu( pMenu );
}

void CSkinnedDialog::OnLButtonDblClk( UINT nFlags, CPoint point )
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnLButtonDblClk( nFlags, point );
	else
		CDialog::OnLButtonDblClk( nFlags, point );
}

void CSkinnedDialog::OnNcLButtonDblClk( UINT nHitTest, CPoint point )
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnNcLButtonDblClk( nHitTest, point );
	else
		CDialog::OnNcLButtonDblClk( nHitTest, point );
}

void CSkinnedDialog::OnCaptureChanged( CWnd* pWnd )
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnCaptureChanged( pWnd );
	else
		CDialog::OnCaptureChanged( pWnd );
}

void CSkinnedDialog::OnTimer( UINT_PTR nIDEvent )
{
	if ( m_pswcoCompanionObj )
		m_pswcoCompanionObj->OnTimer( nIDEvent );
	else
		CDialog::OnTimer( nIDEvent );
}

HBRUSH CSkinnedDialog::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
	HBRUSH		hBrush = CDialog::OnCtlColor( pDC, pWnd, nCtlColor );
	if ( m_pswcoCompanionObj )
	{
		HBRUSH		hOverrBrush = m_pswcoCompanionObj->OnCtlColor( pDC, pWnd, nCtlColor );
		if ( hOverrBrush )
			hBrush = hOverrBrush;
	}
	return hBrush;
}
