//
// SkinnedWndCompanionObj.cpp
//

#include "stdafx.h"
#include "SkinnedWndCompanionObj.h"
#include "SkinnedDialog.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define FRMBTN_UPDATEREQ_TIMERID		666

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Enum and Rect Vector describing the layout of a typical skin.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

static enum eLayout
{
	EL_NormalCaption = 0,
	EL_NormalSideUp,
	EL_NormalSideUp2,
	EL_NormalMinimize,
	EL_NormalMaximize,
	EL_NormalRestore,
	EL_NormalClose,
	EL_RedCaption,
	EL_RedMinimize,
	EL_RedMaximize,
	EL_RedRestore,
	EL_RedClose,
	EL_GrayMinimize,
	EL_GrayMaximize,
	EL_GoldCaption,
	EL_GoldSideUp,
	EL_GoldSideUp2,
	EL_GoldMinimize,
	EL_GoldMaximize,
	EL_GoldRestore,
	EL_GoldClose,
	EL_SideLeft,
	EL_SideRight,
	EL_CornerLeftDown,
	EL_SideDown,
	EL_CornerRightDown,
	EL_END_OF_LIST
};

static RECT g_vrctLayout[ EL_END_OF_LIST ] = {
	{ 1, 1, 217, 43 },
	{ 219, 1, 228, 43 },
	{ 230, 1, 241, 43 },
	{ 243, 1, 266, 43 },
	{ 268, 1, 291, 43 },
	{ 293, 1, 316, 43 },
	{ 318, 1, 346, 43 },
	{ 1, 45, 217, 87 },
	{ 243, 45, 266, 87 },
	{ 268, 45, 291, 87 },
	{ 293, 45, 316, 87 },
	{ 318, 45, 346, 87 },
	{ 243, 89, 266, 131 },
	{ 268, 89, 291, 131 },
	{ 1, 133, 217, 175 },
	{ 219, 133, 228, 175 },
	{ 230, 133, 241, 175 },
	{ 243, 133, 266, 175 },
	{ 268, 133, 291, 175 },
	{ 293, 133, 316, 175 },
	{ 318, 133, 346, 175 },
	{ 1, 177, 24, 186 },
	{ 323, 177, 346, 186 },
	{ 1, 188, 24, 210 },
	{ 26, 188, 35, 210 },
	{ 323, 188, 346, 210 }
};

#define SKINELEM_WIDTH( elElem ) ( g_vrctLayout[ elElem ].right - g_vrctLayout[ elElem ].left + 1 )
#define SKINELEM_HEIGHT( elElem ) ( g_vrctLayout[ elElem ].bottom - g_vrctLayout[ elElem ].top + 1 )

const DWORD g_dwMinSkinSizeX =
	SKINELEM_WIDTH( EL_NormalCaption ) +
	SKINELEM_WIDTH( EL_NormalSideUp2 ) +
	SKINELEM_WIDTH( EL_NormalMinimize ) +
	SKINELEM_WIDTH( EL_NormalMaximize ) +
	SKINELEM_WIDTH( EL_NormalClose );

const DWORD g_dwMinSkinSizeY =
	SKINELEM_HEIGHT( EL_NormalCaption ) +
	SKINELEM_HEIGHT( EL_CornerLeftDown );

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Class Constructor
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CSkinnedWndCompanionObj::CSkinnedWndCompanionObj( CSkinnedDialog* pdlgSkinnedDialog, UINT nIDSkin, COLORREF clrSkinBkgrd ) :
	m_ecsCaptionState( ECS_Normal ),
	m_efbsFirstButtonState( EFBS_NormalMinimize ),
	m_esbsSecondButtonState( ESBS_NormalMaximize ),
	m_etbsThirdButtonState( ETBS_NormalClose ),
	m_euoUserOperation( EUO_None ),
	m_hbBackgBrush( NULL ),
	m_lMinTrackSizeX( -1 ), m_lMinTrackSizeY( -1 )
{
	// init m_uiepUiElementsPos

	::memset( & m_uiepUiElementsPos, 0, sizeof( m_uiepUiElementsPos ) );

	// save the pointer to the window and the skin id

	m_pdlgSkinnedDialog = pdlgSkinnedDialog;
	m_nIDSkin = nIDSkin;
	m_clrSkinBkgrd = clrSkinBkgrd;

	// create and load the skin bitmap

	m_bmpSkin.LoadBitmap( nIDSkin );

	// create the memory dc

	CWindowDC	dc( NULL );
	m_dcMem.CreateCompatibleDC( &dc );

	// create the background brush

	m_hbBackgBrush = ::CreateSolidBrush( m_clrSkinBkgrd );
}

CSkinnedWndCompanionObj::~CSkinnedWndCompanionObj ()
{
	// free the related resources

	if ( m_hbBackgBrush )
		::DeleteObject( m_hbBackgBrush );
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Initialize method
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CSkinnedWndCompanionObj::Initialize()
{
	// check if the bitmap was loaded

	if ( m_bmpSkin.GetSafeHandle() == NULL )
		return FALSE;

	// check if the dc was created

	if ( m_dcMem.GetSafeHdc() == NULL )
		return FALSE;

	// set the region of the window

	SetWindowRgn();

	// set the state of the window frame buttons

	if ( !(m_pdlgSkinnedDialog->GetStyle() & WS_MINIMIZEBOX) )
		m_efbsFirstButtonState = EFBS_GrayMinimize;

	if ( !(m_pdlgSkinnedDialog->GetStyle() & WS_MAXIMIZEBOX) )
		m_esbsSecondButtonState = ESBS_GrayMaximize;

	// return to the caller

	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Handlers
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CSkinnedWndCompanionObj::OnEraseBkgnd( CDC* pDC )
{
/*	// redraw the background

	CalcWndElemsPos();

	CRect rectClient( m_uiepUiElementsPos.rctClientArea );
	m_pdlgSkinnedDialog->ScreenToClient( & rectClient );
	pDC->FillSolidRect( & rectClient, m_clrSkinBkgrd );*/

	// return to the caller

	return TRUE;
}

void CSkinnedWndCompanionObj::OnPaint ()
{
	CPaintDC dc( m_pdlgSkinnedDialog ); // device context for painting

	// redraw the window contents
	RedrawWindow();

	// return to the caller
	return;
}

void CSkinnedWndCompanionObj::OnNcPaint()
{
	// redraw the window contents
	RedrawWindow ( FALSE );

	// return to the caller
	return;
}

void CSkinnedWndCompanionObj::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI)
{
	// set the minimum dimensions of the skinned window
	GetMinTrackSize( lpMMI->ptMinTrackSize.x, lpMMI->ptMinTrackSize.y );

	// return to the caller
	return;
}

void CSkinnedWndCompanionObj::OnSize( UINT nType, int cx, int cy )
{
	// set the region of the window
	SetWindowRgn();

	// return to the caller
	return;
}

UINT CSkinnedWndCompanionObj::OnNcHitTest( CPoint point )
{
	// do the requested operation
	UINT	uiHitTest = HitTest( point );
	return uiHitTest == HTERROR ? HTCAPTION : uiHitTest;
}

BOOL CSkinnedWndCompanionObj::OnNcActivate( BOOL bActive )
{
	// do the requested operation

	if ( bActive )
	{
		m_ecsCaptionState = ECS_Normal;
		UpdateUiElemsState( FALSE, TRUE );
	}
	else
	{
		m_ecsCaptionState = ECS_Gold;
		RedrawWindow( FALSE );
	}

	// return to the caller

	return TRUE;
}

void CSkinnedWndCompanionObj::OnNcLButtonDown( UINT nHitTest, CPoint point )
{
	// call the appropiate handler
	if ( LButtonDown() == FALSE )
		m_pdlgSkinnedDialog->OnNcLButtonDown_CDIALOG( nHitTest, point );
}

void CSkinnedWndCompanionObj::OnNcLButtonUp( UINT nHitTest, CPoint point )
{
	// call the appropiate handler
	if ( LButtonUp() == FALSE )
		m_pdlgSkinnedDialog->OnNcLButtonUp_CDIALOG( nHitTest, point );
}

void CSkinnedWndCompanionObj::OnNcRButtonDown( UINT nHitTest, CPoint point )
{
	// call the appropiate handler
	if ( RButtonDown() == FALSE )
		m_pdlgSkinnedDialog->OnNcRButtonDown_CDIALOG( nHitTest, point );
}

void CSkinnedWndCompanionObj::OnNcRButtonUp( UINT nHitTest, CPoint point )
{
	// call the appropiate handler
	if ( RButtonUp() == FALSE )
		m_pdlgSkinnedDialog->OnNcRButtonUp_CDIALOG( nHitTest, point );
}

void CSkinnedWndCompanionObj::OnLButtonDown( UINT nFlags, CPoint point )
{
	// call the appropiate handler
	if ( LButtonDown() == FALSE )
		m_pdlgSkinnedDialog->OnLButtonDown_CDIALOG( nFlags, point );
}

void CSkinnedWndCompanionObj::OnLButtonUp( UINT nFlags, CPoint point )
{
	// call the appropiate handler
	if ( LButtonUp() == FALSE )
		m_pdlgSkinnedDialog->OnLButtonUp_CDIALOG( nFlags, point );
}

void CSkinnedWndCompanionObj::OnRButtonDown( UINT nFlags, CPoint point )
{
	// call the appropiate handler
	if ( RButtonDown() == FALSE )
		m_pdlgSkinnedDialog->OnRButtonDown_CDIALOG( nFlags, point );
}

void CSkinnedWndCompanionObj::OnRButtonUp( UINT nFlags, CPoint point )
{
	// call the appropiate handler
	if ( RButtonUp() == FALSE )
		m_pdlgSkinnedDialog->OnRButtonUp_CDIALOG( nFlags, point );
}

void CSkinnedWndCompanionObj::OnMouseMove( UINT nFlags, CPoint point )
{
	// call the appropiate handler
	if ( MouseMove() == FALSE )
		m_pdlgSkinnedDialog->OnMouseMove_CDIALOG( nFlags, point );
}

void CSkinnedWndCompanionObj::OnNcMouseMove( UINT nHitTest, CPoint point )
{
	// call the appropiate handler
	if ( MouseMove() == FALSE )
		m_pdlgSkinnedDialog->OnNcMouseMove_CDIALOG( nHitTest, point );
}

void CSkinnedWndCompanionObj::OnLButtonDblClk( UINT nFlags, CPoint point )
{
	// call the appropiate handler
	if ( LButtonDblClk() == FALSE )
		m_pdlgSkinnedDialog->OnLButtonDblClk_CDIALOG( nFlags, point );
}

void CSkinnedWndCompanionObj::OnNcLButtonDblClk( UINT nHitTest, CPoint point )
{
	// call the appropiate handler
	if ( LButtonDblClk() == FALSE )
		m_pdlgSkinnedDialog->OnNcLButtonDblClk_CDIALOG( nHitTest, point );
}

BOOL CSkinnedWndCompanionObj::OnSetCursor( CWnd* pWnd, UINT nHitTest, UINT message )
{
	// do the requested operation

	CPoint		ptCursor;
	::GetCursorPos ( &ptCursor );
	UINT		uiHitTest = HitTest( ptCursor );
	if ( uiHitTest == HTCLIENT )
	{
		return m_pdlgSkinnedDialog->OnSetCursor_CDIALOG( pWnd, nHitTest, message );
	}
	else if ( m_pdlgSkinnedDialog->IsIconic() || m_pdlgSkinnedDialog->IsZoomed() )
	{
		::SetCursor( ::LoadCursor( NULL, IDC_ARROW ) );
		return TRUE;
	}

	BOOL		bResizable = ( m_pdlgSkinnedDialog->GetStyle() & WS_THICKFRAME ) ? TRUE : FALSE;

	switch( uiHitTest )
	{
	case HTTOP:
	case HTBOTTOM:
		::SetCursor( ::LoadCursor( NULL, bResizable ? IDC_SIZENS : IDC_ARROW ) );
		return TRUE;
	case HTLEFT:
	case HTRIGHT:
		::SetCursor( ::LoadCursor( NULL, bResizable ? IDC_SIZEWE : IDC_ARROW ) );
		return TRUE;
	case HTTOPLEFT:
	case HTBOTTOMRIGHT:
		::SetCursor( ::LoadCursor( NULL, bResizable ? IDC_SIZENWSE : IDC_ARROW ) );
		return TRUE;
	case HTTOPRIGHT:
	case HTBOTTOMLEFT:
		::SetCursor( ::LoadCursor( NULL, bResizable ? IDC_SIZENESW : IDC_ARROW ) );
		return TRUE;
	default:
		::SetCursor( ::LoadCursor( NULL, IDC_ARROW ) );
		return TRUE;
	}
}

void CSkinnedWndCompanionObj::OnInitMenu( CMenu* pMenu )
{
	// redraw the window frame
	RedrawWindow ( FALSE, TRUE );

	// return to the caller
	return;
}

void CSkinnedWndCompanionObj::OnCaptureChanged( CWnd* pWnd )
{
	// release the mouse capture, in the case...

	if ( m_euoUserOperation != EUO_None )
	{
		ReleaseSkinWndCapture( TRUE );
	}

	// return to the caller

	return;
}

void CSkinnedWndCompanionObj::OnTimer( UINT_PTR nIDEvent )
{
	// check if this is a WM_TIMER sent for updating and checking the state of the frame buttons

	if ( nIDEvent == FRMBTN_UPDATEREQ_TIMERID )
	{
		UpdateUiElemsState( TRUE );
	}

	// return to the caller

	return;
}

HBRUSH CSkinnedWndCompanionObj::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
	// Return to the caller.
	// NOTE: returning a NULL brush, causes the caller to return the brush returned by the CDialog implementation.

	pDC->SetBkColor( m_clrSkinBkgrd );
	return m_hbBackgBrush;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DrawSkinnedWndElement: draw an element of a skinned window in the specified position
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSkinnedWndCompanionObj::DrawSkinnedWndElement( CDC* pCdc, DWORD dwElementOrdinal, int x, int y, int w /*= 0*/, int h /*= 0*/ )
{
	// do the requested operation

	pCdc->BitBlt( x, y,
		w ? w : SKINELEM_WIDTH( dwElementOrdinal ),
		h ? h : SKINELEM_HEIGHT( dwElementOrdinal ),
		& m_dcMem,
		g_vrctLayout[ dwElementOrdinal ].left, g_vrctLayout[ dwElementOrdinal ].top,
		SRCCOPY );

	// return to the caller

	return;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SetWindowRgn: set the region of the window
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSkinnedWndCompanionObj::SetWindowRgn ( void )
{
	// do the requested operation

	RECT		rctWindow;
	m_pdlgSkinnedDialog->GetWindowRect( & rctWindow );
	long		lWidth = rctWindow.right - rctWindow.left + 1;
	long		lHeight = rctWindow.bottom - rctWindow.top + 1;

	HRGN		hWinRgn = ::CreateRoundRectRgn( 0, 0, lWidth, lHeight, 28, 28 );
	m_pdlgSkinnedDialog->SetWindowRgn( hWinRgn, TRUE );

	// return to the caller

	return;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// RedrawWindow: redraw the window contents
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

static BOOL CALLBACK RedrawChildFunc( HWND hwnd, LPARAM )
{
	::RedrawWindow( hwnd, NULL, NULL, RDW_INVALIDATE | RDW_FRAME | RDW_UPDATENOW );
	return TRUE;
}

void CSkinnedWndCompanionObj::RedrawWindow ( BOOL bRedrawBody /*= TRUE*/, BOOL bRedrawOnlyCaption /*= FALSE*/ )
{
	// check the correctness of the type of the second button

	if ( m_pdlgSkinnedDialog->IsZoomed() )
	{
		if ( m_esbsSecondButtonState == ESBS_NormalMaximize )
			m_esbsSecondButtonState = ESBS_NormalRestore;
	}
	else
	{
		if ( m_esbsSecondButtonState == ESBS_NormalRestore )
			m_esbsSecondButtonState = ESBS_NormalMaximize;
	}

	// paint the skinned frame

	RECT		rctWindow;
	m_pdlgSkinnedDialog->GetWindowRect( & rctWindow );
	long		lWidth = rctWindow.right - rctWindow.left;
	long		lHeight = rctWindow.bottom - rctWindow.top;
	if ( lWidth < (long) g_dwMinSkinSizeX ||
		lHeight < (long) g_dwMinSkinSizeY )
			return;

	CWindowDC dc( m_pdlgSkinnedDialog );
	m_dcMem.SelectObject( & m_bmpSkin );

	// draw the caption

	eLayout		elCaptionType;
	switch( m_ecsCaptionState )
	{
	case ECS_Normal:
		elCaptionType = EL_NormalCaption;
		break;
	case ECS_Red:
		elCaptionType = EL_RedCaption;
		break;
	case ECS_Gold:
		elCaptionType = EL_GoldCaption;
		break;
	}

	DrawSkinnedWndElement( &dc, elCaptionType, 0, 0 );

	eLayout		elSideUpType, elSideUp2Type;
	switch( m_ecsCaptionState )
	{
	case ECS_Gold:
		elSideUpType = EL_GoldSideUp;
		elSideUp2Type = EL_GoldSideUp2;
		break;
	default:
		elSideUpType = EL_NormalSideUp;
		elSideUp2Type = EL_NormalSideUp2;
		break;
	}

	int			iSideUpWidth = lWidth - g_dwMinSkinSizeX;

	int			x = SKINELEM_WIDTH( EL_NormalCaption );
	for ( int i = 0, ic = iSideUpWidth / SKINELEM_WIDTH( elSideUpType ); i < ic; i++ )
	{
		DrawSkinnedWndElement( &dc, elSideUpType, x, 0 );
		x += SKINELEM_WIDTH( elSideUpType );
	}

	int			iSideUpRemainder = iSideUpWidth % SKINELEM_WIDTH( elSideUpType );
	if ( iSideUpRemainder )
	{
		DrawSkinnedWndElement( &dc, elSideUpType, x, 0, iSideUpRemainder );
		x += iSideUpRemainder;
	}

	DrawSkinnedWndElement( &dc, elSideUp2Type, x, 0 );
	x += SKINELEM_WIDTH( elSideUp2Type );

	eLayout		elFirstButtonType;
	switch( m_efbsFirstButtonState )
	{
	case EFBS_NormalMinimize:
		elFirstButtonType = EL_NormalMinimize;
		break;
	case EFBS_RedMinimize:
		elFirstButtonType = EL_RedMinimize;
		break;
	case EFBS_GrayMinimize:
		elFirstButtonType = EL_GrayMinimize;
		break;
	case EFBS_GoldMinimize:
		elFirstButtonType = EL_GoldMinimize;
		break;
	}
	DrawSkinnedWndElement( &dc, elFirstButtonType, x, 0 );
	x += SKINELEM_WIDTH( elFirstButtonType );

	eLayout		elSecondButtonType;
	switch( m_esbsSecondButtonState )
	{
	case ESBS_NormalMaximize:
		elSecondButtonType = EL_NormalMaximize;
		break;
	case ESBS_RedMaximize:
		elSecondButtonType = EL_RedMaximize;
		break;
	case ESBS_GrayMaximize:
		elSecondButtonType = EL_GrayMaximize;
		break;
	case ESBS_GoldMaximize:
		elSecondButtonType = EL_GoldMaximize;
		break;
	case ESBS_NormalRestore:
		elSecondButtonType = EL_NormalRestore;
		break;
	case ESBS_RedRestore:
		elSecondButtonType = EL_RedRestore;
		break;
	case ESBS_GoldRestore:
		elSecondButtonType = EL_GoldRestore;
		break;
	}
	DrawSkinnedWndElement( &dc, elSecondButtonType, x, 0 );
	x += SKINELEM_WIDTH( elSecondButtonType );

	eLayout		elThirdButtonType;
	switch( m_etbsThirdButtonState )
	{
	case ETBS_NormalClose:
		elThirdButtonType = EL_NormalClose;
		break;
	case ETBS_RedClose:
		elThirdButtonType = EL_RedClose;
		break;
	case ETBS_GoldClose:
		elThirdButtonType = EL_GoldClose;
		break;
	}
	DrawSkinnedWndElement( &dc, elThirdButtonType, x, 0 );

	// check if we have to continue

	if ( bRedrawOnlyCaption )
		return;

	// draw the central portion of the window

	int			iSideRightX = lWidth - SKINELEM_WIDTH( EL_SideRight );

	int			iBodyWidth = lWidth
		- SKINELEM_WIDTH( EL_SideLeft ) - SKINELEM_WIDTH( EL_SideRight );

	int			iBodyHeight = lHeight - g_dwMinSkinSizeY;

	int			y = SKINELEM_HEIGHT( EL_NormalCaption );
	for ( int j = 0, jc = iBodyHeight / SKINELEM_HEIGHT( EL_SideLeft ); j < jc; j++ )
	{
		DrawSkinnedWndElement( &dc, EL_SideLeft, 0, y );
		DrawSkinnedWndElement( &dc, EL_SideRight, iSideRightX, y );
		y += SKINELEM_HEIGHT( EL_SideLeft );
	}

	int			iBodyRemainder = iBodyHeight % SKINELEM_HEIGHT( EL_SideLeft );
	if ( iBodyRemainder )
	{
		DrawSkinnedWndElement( &dc, EL_SideLeft, 0, y, 0, iBodyRemainder );
		DrawSkinnedWndElement( &dc, EL_SideRight, iSideRightX, y, 0, iBodyRemainder );
		y += iBodyRemainder;
	}

	if ( bRedrawBody )
	{
		dc.FillSolidRect(
				SKINELEM_WIDTH( EL_SideLeft ), SKINELEM_HEIGHT( EL_NormalCaption ),
				iBodyWidth, iBodyHeight,
				m_clrSkinBkgrd
			);

		// redraw the window children

		::EnumChildWindows( m_pdlgSkinnedDialog->GetSafeHwnd(), & RedrawChildFunc, 0 );
	}

	// draw the bottom portion of the window

	DrawSkinnedWndElement( &dc, EL_CornerLeftDown, 0, y );

	int			iSideDownWidth = lWidth
		- SKINELEM_WIDTH( EL_CornerLeftDown ) - SKINELEM_WIDTH( EL_CornerRightDown );

	x = SKINELEM_WIDTH( EL_CornerLeftDown );
	for ( i = 0, ic = iSideDownWidth / SKINELEM_WIDTH( EL_SideDown ); i < ic; i++ )
	{
		DrawSkinnedWndElement( &dc, EL_SideDown, x, y );
		x += SKINELEM_WIDTH( EL_SideDown );
	}

	int			iSideDownRemainder = iSideDownWidth % SKINELEM_WIDTH( EL_SideDown );
	if ( iSideDownRemainder )
	{
		DrawSkinnedWndElement( &dc, EL_SideDown, x, y, iSideDownRemainder );
		x += iSideDownRemainder;
	}

	DrawSkinnedWndElement( &dc, EL_CornerRightDown, x, y );

	// return to the caller

	return;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CalcWndElemsPos: recalc the positions of every ui element of the skinned window.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSkinnedWndCompanionObj::CalcWndElemsPos ( void )
{
	// do the requested operation

	RECT		rctWindow;
	m_pdlgSkinnedDialog->GetWindowRect( & rctWindow );
	long		lWidth = rctWindow.right - rctWindow.left;
	long		lHeight = rctWindow.bottom - rctWindow.top;

	int			iSideUpWidth = lWidth - g_dwMinSkinSizeX;

	int			iBodyWidth = lWidth
		- SKINELEM_WIDTH( EL_SideLeft ) - SKINELEM_WIDTH( EL_SideRight );

	int			iBodyHeight = lHeight - g_dwMinSkinSizeY;

	m_uiepUiElementsPos.rctCaption.left = rctWindow.left + 7;
	m_uiepUiElementsPos.rctCaption.top = rctWindow.top + 7;
	m_uiepUiElementsPos.rctCaption.right = m_uiepUiElementsPos.rctCaption.left + iSideUpWidth + 210 + 7;
	m_uiepUiElementsPos.rctCaption.bottom = m_uiepUiElementsPos.rctCaption.top + 17;

	m_uiepUiElementsPos.rctFirstButton.left =
		rctWindow.left + SKINELEM_WIDTH( EL_NormalCaption ) + iSideUpWidth + SKINELEM_WIDTH( EL_NormalSideUp2 ) + 3;
	m_uiepUiElementsPos.rctFirstButton.top = rctWindow.top + 7;
	m_uiepUiElementsPos.rctFirstButton.right = m_uiepUiElementsPos.rctFirstButton.left + 17;
	m_uiepUiElementsPos.rctFirstButton.bottom = m_uiepUiElementsPos.rctFirstButton.top + 17;

	m_uiepUiElementsPos.rctSecondButton.left =
		rctWindow.left + SKINELEM_WIDTH( EL_NormalCaption ) + iSideUpWidth + SKINELEM_WIDTH( EL_NormalSideUp2 ) +
		SKINELEM_WIDTH( EL_NormalMinimize ) + 3;
	m_uiepUiElementsPos.rctSecondButton.top = rctWindow.top + 7;
	m_uiepUiElementsPos.rctSecondButton.right = m_uiepUiElementsPos.rctSecondButton.left + 17;
	m_uiepUiElementsPos.rctSecondButton.bottom = m_uiepUiElementsPos.rctSecondButton.top + 17;

	m_uiepUiElementsPos.rctThirdButton.left =
		rctWindow.left + SKINELEM_WIDTH( EL_NormalCaption ) + iSideUpWidth + SKINELEM_WIDTH( EL_NormalSideUp2 ) +
		SKINELEM_WIDTH( EL_NormalMinimize ) + SKINELEM_WIDTH( EL_NormalMaximize ) + 3;
	m_uiepUiElementsPos.rctThirdButton.top = rctWindow.top + 7;
	m_uiepUiElementsPos.rctThirdButton.right = m_uiepUiElementsPos.rctThirdButton.left + 17;
	m_uiepUiElementsPos.rctThirdButton.bottom = m_uiepUiElementsPos.rctThirdButton.top + 17;

	m_uiepUiElementsPos.rctBorder[ 0 ].left = rctWindow.left;
	m_uiepUiElementsPos.rctBorder[ 0 ].top = rctWindow.top;
	m_uiepUiElementsPos.rctBorder[ 0 ].right = rctWindow.right;
	m_uiepUiElementsPos.rctBorder[ 0 ].bottom = rctWindow.top + 14;

	m_uiepUiElementsPos.rctBorder[ 1 ].left = rctWindow.right - 14;
	m_uiepUiElementsPos.rctBorder[ 1 ].top = rctWindow.top;
	m_uiepUiElementsPos.rctBorder[ 1 ].right = rctWindow.right;
	m_uiepUiElementsPos.rctBorder[ 1 ].bottom = rctWindow.bottom;

	m_uiepUiElementsPos.rctBorder[ 2 ].left = rctWindow.left;
	m_uiepUiElementsPos.rctBorder[ 2 ].top = rctWindow.bottom - 14;
	m_uiepUiElementsPos.rctBorder[ 2 ].right = rctWindow.right;
	m_uiepUiElementsPos.rctBorder[ 2 ].bottom = rctWindow.bottom;

	m_uiepUiElementsPos.rctBorder[ 3 ].left = rctWindow.left;
	m_uiepUiElementsPos.rctBorder[ 3 ].top = rctWindow.top;
	m_uiepUiElementsPos.rctBorder[ 3 ].right = rctWindow.left + 14;
	m_uiepUiElementsPos.rctBorder[ 3 ].bottom = rctWindow.bottom;

	m_uiepUiElementsPos.rctClientArea.left = rctWindow.left + SKINELEM_WIDTH( EL_SideLeft );
	m_uiepUiElementsPos.rctClientArea.top = rctWindow.top + SKINELEM_HEIGHT( EL_NormalCaption );
	m_uiepUiElementsPos.rctClientArea.right = m_uiepUiElementsPos.rctClientArea.left + iBodyWidth;
	m_uiepUiElementsPos.rctClientArea.bottom = m_uiepUiElementsPos.rctClientArea.top + iBodyHeight;

	// return to the caller

	return;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// HitTest: test a point against the positions of each UI element.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

UINT CSkinnedWndCompanionObj::HitTest ( CPoint point, BOOL* pbIsOnCloseBtn /*= NULL*/ )
{
	if ( pbIsOnCloseBtn )
		*pbIsOnCloseBtn = FALSE;

	// do the requested operation

	CalcWndElemsPos();

	if ( ::PtInRect( & m_uiepUiElementsPos.rctCaption, point ) )
		return HTCAPTION;
	if ( ::PtInRect( & m_uiepUiElementsPos.rctFirstButton, point ) )
		return HTMINBUTTON;
	if ( ::PtInRect( & m_uiepUiElementsPos.rctSecondButton, point ) )
		return HTMAXBUTTON;
	if ( ::PtInRect( & m_uiepUiElementsPos.rctClientArea, point ) )
		return HTCLIENT;
	if ( ::PtInRect( & m_uiepUiElementsPos.rctThirdButton, point ) )
	{
		if ( pbIsOnCloseBtn )
			*pbIsOnCloseBtn = TRUE;
		return HTERROR;
	}

	DWORD		dwBorderTest = 0;
	if ( ::PtInRect( & m_uiepUiElementsPos.rctBorder[ 0 ], point ) )
		dwBorderTest |= 1;
	if ( ::PtInRect( & m_uiepUiElementsPos.rctBorder[ 1 ], point ) )
		dwBorderTest |= 2;
	if ( ::PtInRect( & m_uiepUiElementsPos.rctBorder[ 2 ], point ) )
		dwBorderTest |= 4;
	if ( ::PtInRect( & m_uiepUiElementsPos.rctBorder[ 3 ], point ) )
		dwBorderTest |= 8;

	switch( dwBorderTest )
	{
	case 1:
		return HTTOP;
	case 2:
		return HTRIGHT;
	case 4:
		return HTBOTTOM;
	case 8:
		return HTLEFT;
	case 1 | 8:
		return HTTOPLEFT;
	case 1 | 2:
		return HTTOPRIGHT;
	case 2 | 4:
		return HTBOTTOMRIGHT;
	case 4 | 8:
		return HTBOTTOMLEFT;
	case 0:
	default:
		return HTERROR;
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// LButtonDown
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CSkinnedWndCompanionObj::LButtonDown ( void )
{
	BOOL		bReturnValue = TRUE;

	// check if we have to begin something

	if ( m_euoUserOperation == EUO_None )
	{
		CPoint		ptCursor;
		::GetCursorPos ( &ptCursor );
		BOOL		bIsOnCloseBtn;
		UINT		uiHitTest = HitTest( ptCursor, &bIsOnCloseBtn );

		BOOL		bUpdateUiElemsState = FALSE;

		BOOL		bResizable = ( m_pdlgSkinnedDialog->GetStyle() & WS_THICKFRAME ) ? TRUE : FALSE;

		switch( uiHitTest )
		{
		case HTCLIENT:
			{
				bReturnValue = FALSE;
			}
			break;
		case HTMINBUTTON:
			{
				if ( m_efbsFirstButtonState != EFBS_GrayMinimize )
				{
					StartUserOperation( EUO_MinimizeBoxClicking );
					m_pdlgSkinnedDialog->SetCapture();
					bUpdateUiElemsState = TRUE;
				}
			}
			break;
		case HTMAXBUTTON:
			{
				if ( m_pdlgSkinnedDialog->IsZoomed() ) // RESTORE
				{
					StartUserOperation( EUO_RestoreBoxClicking );
					m_pdlgSkinnedDialog->SetCapture();
					bUpdateUiElemsState = TRUE;
				}
				else // MAXIMIZE
				{
					if ( m_esbsSecondButtonState != ESBS_GrayMaximize )
					{
						StartUserOperation( EUO_MaximizeBoxClicking );
						m_pdlgSkinnedDialog->SetCapture();
						bUpdateUiElemsState = TRUE;
					}
				}
			}
			break;
		case HTTOP:
			if ( m_pdlgSkinnedDialog->IsZoomed() == FALSE && bResizable )
			{
				m_pdlgSkinnedDialog->SetForegroundWindow();

				StartUserOperation( EUO_SizeNorth );
				m_pdlgSkinnedDialog->SetCapture();
				bUpdateUiElemsState = TRUE;
			}
			break;
		case HTBOTTOM:
			if ( m_pdlgSkinnedDialog->IsZoomed() == FALSE && bResizable )
			{
				m_pdlgSkinnedDialog->SetForegroundWindow();

				StartUserOperation( EUO_SizeSouth );
				m_pdlgSkinnedDialog->SetCapture();
				bUpdateUiElemsState = TRUE;
			}
			break;
		case HTLEFT:
			if ( m_pdlgSkinnedDialog->IsZoomed() == FALSE && bResizable )
			{
				m_pdlgSkinnedDialog->SetForegroundWindow();

				StartUserOperation( EUO_SizeWest );
				m_pdlgSkinnedDialog->SetCapture();
				bUpdateUiElemsState = TRUE;
			}
			break;
		case HTRIGHT:
			if ( m_pdlgSkinnedDialog->IsZoomed() == FALSE && bResizable )
			{
				m_pdlgSkinnedDialog->SetForegroundWindow();

				StartUserOperation( EUO_SizeEast );
				m_pdlgSkinnedDialog->SetCapture();
				bUpdateUiElemsState = TRUE;
			}
			break;
		case HTTOPLEFT:
			if ( m_pdlgSkinnedDialog->IsZoomed() == FALSE && bResizable )
			{
				m_pdlgSkinnedDialog->SetForegroundWindow();

				StartUserOperation( EUO_SizeNorthWest );
				m_pdlgSkinnedDialog->SetCapture();
				bUpdateUiElemsState = TRUE;
			}
			break;
		case HTBOTTOMRIGHT:
			if ( m_pdlgSkinnedDialog->IsZoomed() == FALSE && bResizable )
			{
				m_pdlgSkinnedDialog->SetForegroundWindow();

				StartUserOperation( EUO_SizeSouthEast );
				m_pdlgSkinnedDialog->SetCapture();
				bUpdateUiElemsState = TRUE;
			}
			break;
		case HTTOPRIGHT:
			if ( m_pdlgSkinnedDialog->IsZoomed() == FALSE && bResizable )
			{
				m_pdlgSkinnedDialog->SetForegroundWindow();

				StartUserOperation( EUO_SizeNorthEast );
				m_pdlgSkinnedDialog->SetCapture();
				bUpdateUiElemsState = TRUE;
			}
			break;
		case HTBOTTOMLEFT:
			if ( m_pdlgSkinnedDialog->IsZoomed() == FALSE && bResizable )
			{
				m_pdlgSkinnedDialog->SetForegroundWindow();

				StartUserOperation( EUO_SizeSouthWest );
				m_pdlgSkinnedDialog->SetCapture();
				bUpdateUiElemsState = TRUE;
			}
			break;
		default:
			if ( bIsOnCloseBtn )
			{
				StartUserOperation( EUO_CloseBoxClicking );
				m_pdlgSkinnedDialog->SetCapture();
				bUpdateUiElemsState = TRUE;
			}
			else if ( uiHitTest == HTERROR || uiHitTest == HTCAPTION )
			{
				m_pdlgSkinnedDialog->SetForegroundWindow();

				if ( m_pdlgSkinnedDialog->IsZoomed() == FALSE )
				{
					StartUserOperation( EUO_Move );
					m_pdlgSkinnedDialog->SetCapture();
					bUpdateUiElemsState = TRUE;
				}
			}
			break;
		}

		if ( bUpdateUiElemsState )
		{
			UpdateUiElemsState();
		}
	}

	// return to the caller

	return bReturnValue;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// LButtonUp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CSkinnedWndCompanionObj::LButtonUp ( void )
{
	// check which value to give to the return value

	BOOL		bReturnValue;

	CPoint		ptCursor;
	::GetCursorPos ( &ptCursor );
	BOOL		bIsOnCloseBtn;
	UINT		uiHitTest = HitTest( ptCursor, &bIsOnCloseBtn );

	if ( uiHitTest == HTCLIENT )
		bReturnValue = FALSE;
	else
		bReturnValue = TRUE;

	// check if we have to end something

	if ( m_euoUserOperation != EUO_None )
	{
		ReleaseSkinWndCapture( FALSE );
		RedrawWindow( FALSE, TRUE );

		bReturnValue = TRUE;
	}

	// return to the caller

	return bReturnValue;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// RButtonDown
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CSkinnedWndCompanionObj::RButtonDown ( void )
{
	BOOL		bReturnValue = TRUE;

	// determine the operation to perform

	if ( m_euoUserOperation == EUO_None )
	{
		CPoint		ptCursor;
		::GetCursorPos ( &ptCursor );
		BOOL		bIsOnCloseBtn;
		UINT		uiHitTest = HitTest( ptCursor, &bIsOnCloseBtn );

		if ( uiHitTest == HTCAPTION )
		{
			StartUserOperation( EUO_RightClickOnCaption );
			m_pdlgSkinnedDialog->SetCapture();

			m_ecsCaptionState = ECS_Normal;
			UpdateUiElemsState( FALSE, TRUE );
		}
		else if ( uiHitTest == HTCLIENT )
		{
			bReturnValue = FALSE;
		}
	}

	// return to the caller

	return bReturnValue;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// RButtonUp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CSkinnedWndCompanionObj::RButtonUp ( void )
{
	// check which value to give to the return value

	BOOL		bReturnValue;

	CPoint		ptCursor;
	::GetCursorPos ( &ptCursor );
	BOOL		bIsOnCloseBtn;
	UINT		uiHitTest = HitTest( ptCursor, &bIsOnCloseBtn );

	if ( uiHitTest == HTCLIENT )
		bReturnValue = FALSE;
	else
		bReturnValue = TRUE;

	// check if we have to end something

	if ( m_euoUserOperation == EUO_RightClickOnCaption )
	{
		ReleaseSkinWndCapture( FALSE );

		bReturnValue = TRUE;
	}

	// return to the caller

	return bReturnValue;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MouseMove
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CSkinnedWndCompanionObj::MouseMove ( void )
{
	// check if we have to delegate the requested operation

	if ( m_euoUserOperation != EUO_Move &&
		m_euoUserOperation != EUO_SizeNorth &&
		m_euoUserOperation != EUO_SizeSouth &&
		m_euoUserOperation != EUO_SizeWest &&
		m_euoUserOperation != EUO_SizeEast &&
		m_euoUserOperation != EUO_SizeNorthWest &&
		m_euoUserOperation != EUO_SizeNorthEast &&
		m_euoUserOperation != EUO_SizeSouthWest &&
		m_euoUserOperation != EUO_SizeSouthEast )
	{
		return UpdateUiElemsState();
	}
	else // TRACKING OPERATION
	{
		// accomplish the current operation requirements

		CPoint		ptCursor;
		::GetCursorPos ( &ptCursor );

		int		iX = -99999, iY = -99999, iW = -99999, iH = -99999;

		switch( m_euoUserOperation )
		{
		case EUO_Move:
			{
				iX = ptCursor.x - ( m_ptUserOpCursorStart.x - m_rctUserOpRectStart.left );
				iY = ptCursor.y - ( m_ptUserOpCursorStart.y - m_rctUserOpRectStart.top );
			}
			break;
		case EUO_SizeNorth:
			{
				int		offsetY = m_ptUserOpCursorStart.y - m_rctUserOpRectStart.top;
				iW = m_rctUserOpRectStart.right - m_rctUserOpRectStart.left;
				iH = m_rctUserOpRectStart.bottom - ptCursor.y + offsetY;
				iX = m_rctUserOpRectStart.left;
				iY = ptCursor.y - offsetY;
			}
			break;
		case EUO_SizeSouth:
			{
				int		offsetY = m_rctUserOpRectStart.bottom - m_ptUserOpCursorStart.y;
				iW = m_rctUserOpRectStart.right - m_rctUserOpRectStart.left;
				iH = ptCursor.y - m_rctUserOpRectStart.top + offsetY;
				iX = m_rctUserOpRectStart.left;
				iY = m_rctUserOpRectStart.top;
			}
			break;
		case EUO_SizeWest:
			{
				int		offsetX = m_ptUserOpCursorStart.x - m_rctUserOpRectStart.left;
				iW = m_rctUserOpRectStart.right - ptCursor.x + offsetX;
				iH = m_rctUserOpRectStart.bottom - m_rctUserOpRectStart.top;
				iX = ptCursor.x - offsetX;
				iY = m_rctUserOpRectStart.top;
			}
			break;
		case EUO_SizeEast:
			{
				int		offsetX = m_rctUserOpRectStart.right - m_ptUserOpCursorStart.x;
				iW = ptCursor.x - m_rctUserOpRectStart.left + offsetX;
				iH = m_rctUserOpRectStart.bottom - m_rctUserOpRectStart.top;
				iX = m_rctUserOpRectStart.left;
				iY = m_rctUserOpRectStart.top;
			}
			break;
		case EUO_SizeNorthWest:
			{
				int		offsetX = m_ptUserOpCursorStart.x - m_rctUserOpRectStart.left;
				int		offsetY = m_ptUserOpCursorStart.y - m_rctUserOpRectStart.top;
				iW = m_rctUserOpRectStart.right - ptCursor.x + offsetX;
				iH = m_rctUserOpRectStart.bottom - ptCursor.y + offsetY;
				iX = ptCursor.x - offsetX;
				iY = ptCursor.y - offsetY;
			}
			break;
		case EUO_SizeNorthEast:
			{
				int		offsetX = m_rctUserOpRectStart.right - m_ptUserOpCursorStart.x;
				int		offsetY = m_ptUserOpCursorStart.y - m_rctUserOpRectStart.top;
				iW = ptCursor.x - m_rctUserOpRectStart.left + offsetX;
				iH = m_rctUserOpRectStart.bottom - ptCursor.y + offsetY;
				iX = m_rctUserOpRectStart.left;
				iY = ptCursor.y - offsetY;
			}
			break;
		case EUO_SizeSouthWest:
			{
				int		offsetX = m_ptUserOpCursorStart.x - m_rctUserOpRectStart.left;
				int		offsetY = m_rctUserOpRectStart.bottom - m_ptUserOpCursorStart.y;
				iW = m_rctUserOpRectStart.right - ptCursor.x + offsetX;
				iH = ptCursor.y - m_rctUserOpRectStart.top + offsetY;
				iX = ptCursor.x - offsetX;
				iY = m_rctUserOpRectStart.top;
			}
			break;
		case EUO_SizeSouthEast:
			{
				int		offsetX = m_rctUserOpRectStart.right - m_ptUserOpCursorStart.x;
				int		offsetY = m_rctUserOpRectStart.bottom - m_ptUserOpCursorStart.y;
				iW = ptCursor.x - m_rctUserOpRectStart.left + offsetX;
				iH = ptCursor.y - m_rctUserOpRectStart.top + offsetY;
				iX = m_rctUserOpRectStart.left;
				iY = m_rctUserOpRectStart.top;
			}
			break;
		}

		if ( iW != -99999 )
		{
			POINT		ptMinTrackSize;
			GetMinTrackSize( ptMinTrackSize.x, ptMinTrackSize.y );

			if ( iW < ptMinTrackSize.x )
			{
				if ( m_euoUserOperation == EUO_SizeNorth ||
					m_euoUserOperation == EUO_SizeWest ||
					m_euoUserOperation == EUO_SizeNorthWest ||
					m_euoUserOperation == EUO_SizeSouthWest )
						iX -= ptMinTrackSize.x - iW;

				iW = ptMinTrackSize.x;
			}
			if ( iH < ptMinTrackSize.y )
			{
				if ( m_euoUserOperation == EUO_SizeNorth ||
					m_euoUserOperation == EUO_SizeWest ||
					m_euoUserOperation == EUO_SizeNorthWest ||
					m_euoUserOperation == EUO_SizeNorthEast )
						iY -= ptMinTrackSize.y - iH;

				iH = ptMinTrackSize.y;
			}
		}

		m_pdlgSkinnedDialog->SetWindowPos( NULL,
			iX, iY,
			iW, iH,
			( iX == -99999 ? SWP_NOMOVE : 0 ) | ( iW == -99999 ? SWP_NOSIZE : 0 ) | SWP_NOZORDER );

		// return to the caller

		return TRUE;
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// LButtonDblClk
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CSkinnedWndCompanionObj::LButtonDblClk ( void )
{
	// determine the operation to perform

	if ( m_euoUserOperation == EUO_None )
	{
		CPoint		ptCursor;
		::GetCursorPos ( &ptCursor );
		BOOL		bIsOnCloseBtn;
		UINT		uiHitTest = HitTest( ptCursor, &bIsOnCloseBtn );

		if ( uiHitTest == HTERROR || uiHitTest == HTCAPTION )
		{
			if ( m_pdlgSkinnedDialog->IsZoomed() )
			{
				m_pdlgSkinnedDialog->SendMessage( WM_SYSCOMMAND, SC_RESTORE );
			}
			else
			{
				if ( m_esbsSecondButtonState != ESBS_GrayMaximize )
					m_pdlgSkinnedDialog->SendMessage( WM_SYSCOMMAND, SC_MAXIMIZE );
			}
		}
	}

	// return to the caller

	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// UpdateUiElemsState: according to the mouse cursor position, update the state of the skinned frame elements.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CSkinnedWndCompanionObj::UpdateUiElemsState ( BOOL bCalledFromTimerHandler /*= FALSE*/, BOOL bRedrawHowever /*= FALSE*/ )
{
	BOOL		bReturnValue = TRUE;

	// do the requested operation

	CPoint		ptCursor;
	::GetCursorPos ( &ptCursor );
	BOOL		bIsOnCloseBtn;
	UINT		uiHitTest = HitTest( ptCursor, &bIsOnCloseBtn );

	BOOL		bCursorInWindow = ( ::WindowFromPoint( ptCursor ) == m_pdlgSkinnedDialog->m_hWnd );

	BOOL		bForceRedraw = bRedrawHowever;
	BOOL		bTimerNeeded = FALSE;

	// ### check the active state of the elements ###

	switch ( uiHitTest )
	{
	case HTCLIENT:
		{
			if ( m_euoUserOperation == EUO_None )
			{
				bReturnValue = FALSE;
			}
		}
		break;
	case HTCAPTION: // CAPTION
		{
			if ( m_euoUserOperation == EUO_None )
			{
				if ( bCursorInWindow == FALSE )
				{
					m_ecsCaptionState = ECS_Normal;
					bForceRedraw = TRUE;
				}
				else if ( m_ecsCaptionState == ECS_Normal )
				{
					m_ecsCaptionState = ECS_Red;
					bForceRedraw = TRUE;
					bTimerNeeded = TRUE;
				}
				else if ( m_ecsCaptionState == ECS_Red )
				{
					bTimerNeeded = TRUE;
				}
			}
		}
		break;
	case HTMINBUTTON: // MINIMIZE BOX
		{
			if ( m_euoUserOperation == EUO_None )
			{
				if ( bCursorInWindow == FALSE )
				{
					m_efbsFirstButtonState = EFBS_NormalMinimize;
					bForceRedraw = TRUE;
				}
				else if ( m_efbsFirstButtonState == EFBS_NormalMinimize )
				{
					m_efbsFirstButtonState = EFBS_RedMinimize;
					bForceRedraw = TRUE;
					bTimerNeeded = TRUE;
				}
				else if ( m_efbsFirstButtonState == EFBS_GoldMinimize )
				{
					m_efbsFirstButtonState = EFBS_NormalMinimize;
					bForceRedraw = TRUE;
				}
				else if ( m_efbsFirstButtonState == EFBS_RedMinimize )
				{
					bTimerNeeded = TRUE;
				}
			}
			else if ( m_euoUserOperation == EUO_MinimizeBoxClicking )
			{
				if ( m_efbsFirstButtonState != EFBS_GoldMinimize )
				{
					m_efbsFirstButtonState = EFBS_GoldMinimize;
					bForceRedraw = TRUE;
				}
			}
		}
		break;
	case HTMAXBUTTON: // MAXIMIZE BOX
		{
			if ( m_euoUserOperation == EUO_None )
			{
				if ( bCursorInWindow == FALSE )
				{
					if ( m_pdlgSkinnedDialog->IsZoomed() )
						m_esbsSecondButtonState = ESBS_NormalRestore;
					else
						m_esbsSecondButtonState = ESBS_NormalMaximize;
					bForceRedraw = TRUE;
				}
				else if ( m_esbsSecondButtonState == ESBS_NormalMaximize )
				{
					m_esbsSecondButtonState = ESBS_RedMaximize;
					bForceRedraw = TRUE;
					bTimerNeeded = TRUE;
				}
				else if ( m_esbsSecondButtonState == ESBS_GoldMaximize )
				{
					m_esbsSecondButtonState = ESBS_NormalMaximize;
					bForceRedraw = TRUE;
				}
				else if ( m_esbsSecondButtonState == ESBS_RedMaximize )
				{
					bTimerNeeded = TRUE;
				}
				else if ( m_esbsSecondButtonState == ESBS_NormalRestore )
				{
					m_esbsSecondButtonState = ESBS_RedRestore;
					bForceRedraw = TRUE;
					bTimerNeeded = TRUE;
				}
				else if ( m_esbsSecondButtonState == ESBS_GoldRestore )
				{
					m_esbsSecondButtonState = ESBS_NormalRestore;
					bForceRedraw = TRUE;
				}
				else if ( m_esbsSecondButtonState == ESBS_RedRestore )
				{
					bTimerNeeded = TRUE;
				}
			}
			else if ( m_euoUserOperation == EUO_MaximizeBoxClicking )
			{
				if ( m_esbsSecondButtonState != ESBS_GoldMaximize )
				{
					m_esbsSecondButtonState = ESBS_GoldMaximize;
					bForceRedraw = TRUE;
				}
			}
			else if ( m_euoUserOperation == EUO_RestoreBoxClicking )
			{
				if ( m_esbsSecondButtonState != ESBS_GoldRestore )
				{
					m_esbsSecondButtonState = ESBS_GoldRestore;
					bForceRedraw = TRUE;
				}
			}
		}
		break;
	default:
		if ( bIsOnCloseBtn ) // CLOSE BOX
		{
			if ( m_euoUserOperation == EUO_None )
			{
				if ( bCursorInWindow == FALSE )
				{
					m_etbsThirdButtonState = ETBS_NormalClose;
					bForceRedraw = TRUE;
				}
				else if ( m_etbsThirdButtonState == ETBS_NormalClose )
				{
					m_etbsThirdButtonState = ETBS_RedClose;
					bForceRedraw = TRUE;
					bTimerNeeded = TRUE;
				}
				else if ( m_etbsThirdButtonState == ETBS_GoldClose )
				{
					m_etbsThirdButtonState = ETBS_NormalClose;
					bForceRedraw = TRUE;
				}
				else if ( m_etbsThirdButtonState == ETBS_RedClose )
				{
					bTimerNeeded = TRUE;
				}
			}
			else if ( m_euoUserOperation == EUO_CloseBoxClicking )
			{
				if ( m_etbsThirdButtonState != ETBS_GoldClose )
				{
					m_etbsThirdButtonState = ETBS_GoldClose;
					bForceRedraw = TRUE;
				}
			}
		}
		break;
	}

	// ### check the passive state of the elements ###

	if ( uiHitTest != HTCAPTION ) // NOT CAPTION
	{
		if ( m_euoUserOperation == EUO_None )
		{
			if ( m_ecsCaptionState == ECS_Red )
			{
				m_ecsCaptionState = ECS_Normal;
				bForceRedraw = TRUE;
			}
		}
		else
		{
			if ( m_ecsCaptionState != ECS_Normal )
			{
				m_ecsCaptionState = ECS_Normal;
				bForceRedraw = TRUE;
			}
		}
	}
	if ( uiHitTest != HTMINBUTTON ) // NOT MINIMIZE BOX
	{
		if ( m_euoUserOperation == EUO_None )
		{
			if ( m_efbsFirstButtonState == EFBS_RedMinimize )
			{
				m_efbsFirstButtonState = EFBS_NormalMinimize;
				bForceRedraw = TRUE;
			}
		}
		else if ( m_efbsFirstButtonState != EFBS_GrayMinimize )
		{
			if ( m_efbsFirstButtonState != EFBS_NormalMinimize )
			{
				m_efbsFirstButtonState = EFBS_NormalMinimize;
				bForceRedraw = TRUE;
			}
		}
	}
	if ( uiHitTest != HTMAXBUTTON ) // NOT MAXIMIZE BOX
	{
		if ( m_euoUserOperation == EUO_None )
		{
			if ( m_esbsSecondButtonState == ESBS_RedMaximize )
			{
				m_esbsSecondButtonState = ESBS_NormalMaximize;
				bForceRedraw = TRUE;
			}
			else if ( m_esbsSecondButtonState == ESBS_RedRestore )
			{
				m_esbsSecondButtonState = ESBS_NormalRestore;
				bForceRedraw = TRUE;
			}
		}
		else
		{
			if ( m_pdlgSkinnedDialog->IsZoomed() )
			{
				if ( m_esbsSecondButtonState != ESBS_NormalRestore )
				{
					m_esbsSecondButtonState = ESBS_NormalRestore;
					bForceRedraw = TRUE;
				}
			}
			else
			{
				if ( m_esbsSecondButtonState != ESBS_GrayMaximize )
				{
					if ( m_esbsSecondButtonState != ESBS_NormalMaximize )
					{
						m_esbsSecondButtonState = ESBS_NormalMaximize;
						bForceRedraw = TRUE;
					}
				}
			}
		}
	}
	if ( bIsOnCloseBtn == FALSE ) // NOT CLOSE BOX
	{
		if ( m_euoUserOperation == EUO_None )
		{
			if ( m_etbsThirdButtonState == ETBS_RedClose )
			{
				m_etbsThirdButtonState = ETBS_NormalClose;
				bForceRedraw = TRUE;
			}
		}
		else
		{
			if ( m_etbsThirdButtonState != ETBS_NormalClose )
			{
				m_etbsThirdButtonState = ETBS_NormalClose;
				bForceRedraw = TRUE;
			}
		}
	}

	// ### check if we have to set or kill the timer ###

	if ( bCalledFromTimerHandler )
	{
		if (	bTimerNeeded == FALSE ||
				bCursorInWindow == FALSE
			)
		{
			m_pdlgSkinnedDialog->KillTimer( FRMBTN_UPDATEREQ_TIMERID );
		}
	}
	else
	{
		if ( bTimerNeeded != FALSE )
		{
			m_pdlgSkinnedDialog->SetTimer( FRMBTN_UPDATEREQ_TIMERID, 1, NULL );
		}
	}

	// ### check if we have to redraw ###

	if ( bForceRedraw )
	{
		RedrawWindow( FALSE, TRUE );
	}

	// return to the caller

	return bReturnValue;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// ReleaseSkinWndCapture: release the mouse capture
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSkinnedWndCompanionObj::ReleaseSkinWndCapture ( BOOL bCaptureChanged )
{
	// do the requested operation

	if ( m_euoUserOperation != EUO_None )
	{
		// set the current operation to NONE

		eUserOperation			euoThis = m_euoUserOperation;

		m_euoUserOperation = EUO_None;

		// release the capture, if not yet done

		::ReleaseCapture();

		// update the frame buttons

		if ( euoThis != EUO_RightClickOnCaption &&
			euoThis != EUO_Move &&
			euoThis != EUO_SizeNorth &&
			euoThis != EUO_SizeSouth &&
			euoThis != EUO_SizeWest &&
			euoThis != EUO_SizeEast &&
			euoThis != EUO_SizeNorthWest &&
			euoThis != EUO_SizeNorthEast &&
			euoThis != EUO_SizeSouthWest &&
			euoThis != EUO_SizeSouthEast )
				UpdateUiElemsState();

		// do the user requested operation

		if ( bCaptureChanged == FALSE )
		{
			// check if the user has released the mouse button on the initial ui element

			CPoint		ptCursor;
			::GetCursorPos ( &ptCursor );
			BOOL		bIsOnCloseBtn;
			UINT		uiHitTest = HitTest( ptCursor, &bIsOnCloseBtn );

			switch( euoThis )
			{
			case EUO_MinimizeBoxClicking:
				if ( uiHitTest == HTMINBUTTON )
				{
					m_pdlgSkinnedDialog->SendMessage( WM_SYSCOMMAND, SC_MINIMIZE );
				}
				break;
			case EUO_MaximizeBoxClicking:
				if ( uiHitTest == HTMAXBUTTON )
				{
					m_pdlgSkinnedDialog->SendMessage( WM_SYSCOMMAND, SC_MAXIMIZE );
				}
				break;
			case EUO_RestoreBoxClicking:
				if ( uiHitTest == HTMAXBUTTON )
				{
					m_pdlgSkinnedDialog->SendMessage( WM_SYSCOMMAND, SC_RESTORE );
				}
				break;
			case EUO_CloseBoxClicking:
				if ( bIsOnCloseBtn )
				{
					m_pdlgSkinnedDialog->SendMessage( WM_SYSCOMMAND, SC_CLOSE );
				}
				break;
			case EUO_RightClickOnCaption:
				if ( uiHitTest == HTCAPTION )
				{
					CMenu*		pmSysMenu = m_pdlgSkinnedDialog->GetSystemMenu( FALSE );
					if ( pmSysMenu )
					{
						if ( !(m_pdlgSkinnedDialog->GetStyle() & WS_MINIMIZEBOX) )
							pmSysMenu->EnableMenuItem( SC_MINIMIZE, MF_GRAYED | MF_BYCOMMAND );
						else
							pmSysMenu->EnableMenuItem( SC_MINIMIZE, MF_ENABLED | MF_BYCOMMAND );

						if ( !(m_pdlgSkinnedDialog->GetStyle() & WS_MAXIMIZEBOX) )
							pmSysMenu->EnableMenuItem( SC_MAXIMIZE, MF_GRAYED | MF_BYCOMMAND );
						else
							pmSysMenu->EnableMenuItem( SC_MAXIMIZE, MF_ENABLED | MF_BYCOMMAND );

						if ( m_pdlgSkinnedDialog->IsZoomed() )
						{
							pmSysMenu->EnableMenuItem( SC_MAXIMIZE, MF_GRAYED | MF_BYCOMMAND );
							pmSysMenu->EnableMenuItem( SC_RESTORE, MF_ENABLED | MF_BYCOMMAND );
							pmSysMenu->EnableMenuItem( SC_MOVE, MF_GRAYED | MF_BYCOMMAND );
							pmSysMenu->EnableMenuItem( SC_SIZE, MF_GRAYED | MF_BYCOMMAND );
						}
						else
						{
							pmSysMenu->EnableMenuItem( SC_MAXIMIZE, MF_ENABLED | MF_BYCOMMAND );
							pmSysMenu->EnableMenuItem( SC_RESTORE, MF_GRAYED | MF_BYCOMMAND );
							pmSysMenu->EnableMenuItem( SC_MOVE, MF_ENABLED | MF_BYCOMMAND );
							pmSysMenu->EnableMenuItem( SC_SIZE, MF_ENABLED | MF_BYCOMMAND );
						}

						DWORD	dwSel = ::TrackPopupMenu( pmSysMenu->m_hMenu, TPM_RETURNCMD,
							ptCursor.x, ptCursor.y,
							0, m_pdlgSkinnedDialog->m_hWnd,
							NULL );

						if ( dwSel )
							m_pdlgSkinnedDialog->PostMessage( WM_SYSCOMMAND, dwSel );
					}
				}
				break;
			}
		}
	}

	// return to the caller

	return;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// StartUserOperation: set the variables for starting a user operation with the mouse.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSkinnedWndCompanionObj::StartUserOperation( eUserOperation euoUserOperation )
{
	// do the requested operation

	::GetCursorPos( & m_ptUserOpCursorStart );
	m_pdlgSkinnedDialog->GetWindowRect( & m_rctUserOpRectStart );

	m_euoUserOperation = euoUserOperation;

	// return to the caller

	return;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GetMinTrackSize: get the min track size dimensions for the skinned window.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSkinnedWndCompanionObj::GetMinTrackSize( long& lX, long& lY )
{
	// return the information
	if ( m_lMinTrackSizeX == -1 )
	{
		lX = g_dwMinSkinSizeX + 64;
		lY = g_dwMinSkinSizeY + 64;
	}
	else
	{
		lX = m_lMinTrackSizeX;
		lY = m_lMinTrackSizeY;
	}

	// return to the caller
	return;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SetMinTrackSize: set the min track size dimensions for the skinned window.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSkinnedWndCompanionObj::SetMinTrackSize( long lX, long lY )
{
	// store the dimensions
	m_lMinTrackSizeX = lX;
	m_lMinTrackSizeY = lY;

	// return to the caller
	return;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GetClientRectInScreenCoords method
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSkinnedWndCompanionObj::GetClientRectInScreenCoords( CRect& rctClientRect )
{
	// return the required information
	CalcWndElemsPos();
	rctClientRect = m_uiepUiElementsPos.rctClientArea;
	return;
}
